﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking;

namespace HeuristicLab.Encodings.PackingEncoding {
  [Item("PackingSolutionCreator", "Represents the generalized form of creators for bin packing problems.")]
  [StorableClass]
  public abstract class PackingSolutionCreator : SingleSuccessorOperator, IPackingSolutionCreator {

    public ILookupParameter<IItem> EncodedSolutionParameter {
      get { return (ILookupParameter<IItem>)Parameters["EncodedPackingSolution"]; }
    }

    [StorableConstructor]
    protected PackingSolutionCreator(bool deserializing) : base(deserializing) { }
    protected PackingSolutionCreator(PackingSolutionCreator original, Cloner cloner) : base(original, cloner) { }
    public PackingSolutionCreator()
      : base() {
      Parameters.Add(new LookupParameter<IItem>("EncodedPackingSolution", "The new bin packing solution candidate."));
    }

    public override IOperation Apply() {
      EncodedSolutionParameter.ActualValue = CreateSolution();
      return base.Apply();
    }

    protected abstract IItem CreateSolution();
  }
}
